@@ -25,6 +25,8 @@
 
    cpu		GRI-909 CPU
 
+   18-Jul-04	RMS	Fixed missing ao_update calls in AX, AY write
+   17-Jul-04	RMS	Revised MSR, EAO based on additional documentation
    14-Mar-03	RMS	Fixed bug in SC queue tracking
 
    The system state for the GRI-909 is:
@@ -380,7 +382,7 @@ extern UNIT rtc_unit;
 
 SC = SC & AMASK;					/* load local PC */
 reason = 0;
-AO = ao_update ();					/* update AO */
+ao_update ();						/* update AO */
 sim_rtc_init (rtc_unit.wait);				/* init calibration */
 
 /* Main instruction fetch/decode loop */
@@ -516,7 +518,7 @@ else {	SC = (SC + 1) & AMASK;				/* incr SC */
 
 /* Simulation halted */
 
-AO = ao_update ();					/* update AO */
+ao_update ();						/* update AO */
 scq_r->qptr = scq_p;					/* update sc q ptr */
 return reason;
 }
@@ -699,12 +701,12 @@ return SWR;
 
 uint32 msr_rd (uint32 src)
 {
-return MSR;
+return MSR & MSR_RW;
 }
 
 t_stat msr_wr (uint32 src, uint32 dat)
 {
-MSR = dat;						/* new MSR */
+MSR = dat & MSR_RW;					/* new MSR */
 ao_update ();						/* update AOV */
 return SCPE_OK;
 }
@@ -713,25 +715,27 @@ return SCPE_OK;
 
 uint32 ao_update (void)
 {
-int32 t;
-int32 af = MSR_GET_FOA (MSR);
+uint32 af = MSR_GET_FOA (MSR);
 
 switch (af) {
 case AO_ADD:
-	t = (AX + AY) & DMASK;				/* add */
+	AO = (AX + AY) & DMASK;				/* add */
 	break;
 case AO_AND:
-	t = AX & AY;					/* and */
+	AO = AX & AY;					/* and */
 	break;
 case AO_XOR:						/* xor */
-	t = AX ^ AY;
+	AO = AX ^ AY;
 	break;
 case AO_IOR:
-	t = AX | AY;					/* or */
+	AO = AX | AY;					/* or */
 	break;  }
 if ((AX + AY) & CBIT) MSR = MSR | MSR_AOV;		/* always calc AOV */
 else MSR = MSR & ~MSR_AOV;
-return t;
+if (SIGN & ((AX ^ (AX + AY)) & (~AX ^ AY)))		/* always calc SOV */
+	MSR = MSR | MSR_SOV;
+else MSR = MSR & ~MSR_SOV;
+return AO;
 }
 
 uint32 ax_rd (uint32 src)
@@ -742,6 +746,7 @@ return AX;
 t_stat ax_wr (uint32 dst, uint32 dat)
 {
 AX = dat;
+ao_update ();
 return SCPE_OK;
 }
 
@@ -753,18 +758,19 @@ return AY;
 t_stat ay_wr (uint32 dst, uint32 dat)
 {
 AY = dat;
+ao_update ();
 return SCPE_OK;
 }
 
 uint32 ao_rd (uint32 src)
 {
-AO = ao_update ();
-return AO;
+return ao_update ();
 }
 
 t_stat ao_fo (uint32 op)
 {
 uint32 t = OP_GET_FOA (op);				/* get func */
+
 MSR = MSR_PUT_FOA (MSR, t);				/* store in MSR */
 ao_update ();						/* update AOV */
 return SCPE_OK;
@@ -778,21 +784,41 @@ if (cpu_unit.flags & UNIT_NOEAO) return stop_opr;	/* EAO installed? */
 if (op == EAO_MUL) {					/* mul? */
 	t = AX * AY;					/* AX * AY */
 	AX = (t >> 16) & DMASK;				/* to AX'GR1 */
-	GR[0] = t & DMASK;  }
-if (op == EAO_DIV) {					/* div? */
+	GR[0] = t & DMASK;
+	}
+else if (op == EAO_DIV) {				/* div? */
 	if (AY && (AX < AY)) {
 	    t = (AX << 16) | GR[0];			/* AX'GR1 / AY */
 	    GR[0] = t / AY;				/* quo to GR1 */
-	    AX = t % AY;  }				/* rem to AX */
+	    AX = t % AY;				/* rem to AX */
+	    MSR = MSR & ~MSR_L;  }			/* clear link */
+	else MSR = MSR | MSR_L;				/* set link */
 	}
+else if (op == EAO_ARS) {				/* arith right? */
+	t = 0;						/* shift limiter */
+	if (AX & SIGN) MSR = MSR | MSR_L;		/* L = sign */
+	else MSR = MSR & ~MSR_L;
+	do {						/* shift one bit */
+	    AY = ((AY >> 1) | (AX << 15)) & DMASK;
+	    AX = (AX & SIGN) | (AX >> 1);
+	    GR[0] = (GR[0] + 1) & DMASK;  }
+	while (GR[0] && (++t < 32));			/* until cnt or limit */
+	}	    
+else if (op == EAO_NORM) {				/* norm? */
+	if ((AX | AY) != 0) {				/* can normalize? */
+	    while ((AX & SIGN) != ((AX << 1) & SIGN)) {	/* until AX15 != AX14 */
+		AX = ((AX << 1) | (AY >> 15)) & DMASK;
+		AY = (AY << 1) & DMASK;
+		GR[0] = (GR[0] + 1) & DMASK;  }  }
+	}
+ao_update ();
 return SCPE_OK;
 }
 
 uint32 ao_sf (uint32 op)
 {
 if (((op & 2) && (MSR & MSR_AOV)) ||			/* arith carry? */
-    ((op & 4) && (SIGN &				/* arith overflow? */
-	((AX ^ (AX + AY)) & (~AX ^ AY))))) return 1;
+    ((op & 4) && (MSR & MSR_SOV))) return 1;		/* arith overflow? */
 return 0;
 }
 
