@@ -23,6 +23,7 @@
    be used in advertising or otherwise to promote the sale, use or other dealings
    in this Software without prior written authorization from Robert M Supnik.
 
+   25-Feb-05	JDB	Added FFP helpers f_pack, f_unpack, f_pwr2
    11-Feb-05	JDB	Fixed missing negative overflow renorm in StoreFP
    26-Dec-04	RMS	Separated A/B from M[0/1] for DMA IO (from Dave Bryan)
    15-Jul-03	RMS	Fixed signed/unsigned warning
@@ -105,6 +106,7 @@ struct ufp {						/* unpacked fp */
 #define FR_NEG(v)	((~(v) + 1) & DMASK32)
 
 extern uint16 ABREG[2];
+
 uint32 UnpackFP (struct ufp *fop, uint32 opnd);
 void NegFP (struct ufp *fop);
 void NormFP (struct ufp *fop);
@@ -260,6 +262,46 @@ if (fop1.fr) {						/* dvd != 0? */
 return StoreFP (&quo);					/* store result */
 }
 
+/* Fast FORTRAN Processor helpers. */
+
+/* Pack mantissa in A/B and exponent and return fp in A/B */
+
+uint32 f_pack (int32 expon)
+{
+struct ufp fop;
+
+fop.fr = FPAB;
+fop.exp = expon;
+return StoreFP (&fop);
+}
+
+/* Unpack fp number in A/B into A (exponent) and B (lower mantissa) */
+
+void f_unpack (void)
+{
+AR = FP_GETEXP (BR);					/* get exponent */
+if (FP_GETEXPS (BR)) AR = (AR | ~FP_M_EXP) & DMASK;	/* < 0? sext */
+BR = BR & (uint16) ~(FP_EXP | FP_EXPS);			/* clear exp */
+return;
+}
+
+/* Multiply fp number in A/B by 2**n and return in A/B.
+   Exponent overflow or underflow wraps around. */
+
+void f_pwr2 (int32 n)
+{
+uint32 save_a;
+
+if (AR | BR) {						/* microcode test */
+	save_a = AR;
+	f_unpack ();					/* unpack exponent */
+	AR = AR + n;					/* multiply */
+	BR = BR | ((AR & FP_M_EXP) << FP_V_EXP) |	/* merge exponent */
+		  ((AR & SIGN)? (1 << FP_V_EXPS): 0);	/* and exponent sign */
+	AR = save_a;  }
+return;
+}
+
 /* Utility routines */
 
 /* Unpack operand */
