@@ -47,7 +47,7 @@
 #elif defined (VM_VAX)					/* VAX version */
 #include "vax_defs.h"
 #if (UNIBUS)
-#define INIT_TYPE	TQU_TYPE
+#define INIT_TYPE	TQ8_TYPE
 #else
 #define INIT_TYPE	TQ5_TYPE
 #endif
@@ -1755,7 +1755,6 @@ return SCPE_OK;
 #define BOOT_CSR	(BOOT_START + 014)		/* CSR */
 #define BOOT_LEN	(sizeof (boot_rom) / sizeof (int16))
 
-
 /* Data structure definitions */
 
 #define B_CMDINT	(BOOT_START - 01000)		/* cmd int */
